<?php
abstract class df_element
{
	protected $elm_id           = NULL;
  protected $elm_name         = NULL;
  protected $elm_type         = NULL;
  protected $elm_value        = NULL;
  protected $elm_label        = NULL;
  protected $elm_tabindex     = NULL;
  protected $elm_jscript      = NULL;
  protected $elm_css_class    = NULL;
  public    $elm_bound        = false;
  public    $elm_bound_to     = NULL;
  protected $elm_html_after   = NULL;
  protected $elm_html_before  = NULL;
  protected $elm_bounded_draw = true;
  
	public function __construct($name, $id=NULL, $value=NULL, $label=NULL, $css_class=NULL, $tabindex=NULL, $jscript=NULL)
	{
	  ($id == NULL) ? ($this->elm_id = $name) : ($this->elm_id = $id);
		$this->elm_name      = $name;
	  $this->elm_css_class = $css_class;
	  $this->elm_label = $label; 
	  $this->setup($value, $tabindex, $jscript);
	}
	
  private function __clone()
  {  
  }
  
  /*
   * Freeing the connection to the element counter
   * and the regarding form
   */
  public function __destruct()
  {
  	 
  }
  
  /*
   * Define here the setup of the more complex and element specific parameters
   */
  protected abstract function setup($value = NULL, $tabindex = NULL, $jscript = NULL);
  /*
   * Define here the output created by the Element
   */
  public abstract function draw();
  
  /*
   * Checking Methods
   */
  protected function isBound()
  {
  	if($this->elm_bound_to != NULL)
  	{
  		return true;
  	}
  	else
  	{
  		return false;
  	}
  }
  protected function isDesignActive()
  {
    if($this->isBound() == true)
    {
      if($this->getForm()->getMain()->hasDesign())
      {
      	return true;
      }
      else
      {
      	return false;
      }
    }
    else
    {
    	return false;
    }
  }
  
  /*
   * Getter & Setter
   */
  public function setBoundedDraw($bool)
  {
  	$this->elm_bounded_draw = $bool;
  }
  public function isBoundedDraw()
  {
  	return $this->elm_bounded_draw; 
  }
  public function setHTMLAfter($value)
  {
  	$this->elm_html_after = $value;
  }
  public function setHTMLBefore($value)
  {
    $this->elm_html_before = $value;
  }
  public function getType()
  {
  	return $this->elm_type;
  }
  public function getName()
  {
  	return $this->elm_name;
  }
  public function getCSS()
  {
  	return $this->elm_css_class;
  }
  public function setCSS($value)
  {
  	$this->elm_css_class = $value;
  }
  public function addCSS($value)
  {
    $this->elm_css_class .= " ".$value;
  }
  public function getTabindex()
  {
  	return $this->elm_tabindex;
  }
  public function getId()
  {
  	return $this->elm_id;
  }
  public function getForm()
  {
  	return $this->elm_bound_to;
  }
}

class df_textbox extends df_element
{
  protected function setup($value = NULL, $tabindex = NULL, $jscript = NULL)
  {
  	$this->elm_type = "textbox";
  	$this->elm_value = $value;
  }
  
  public function draw()
  {
    if($this->isDesignActive())
    {
      $this->addCSS("input_text"); 
    }
  	$output = NULL;
  	$output .= $this->elm_html_before;
  	
  	if($this->elm_label != NULL)
  	{
  		$output .= "<label for=\"".$this->elm_id."\">".$this->elm_label."</label>";
  	}
  	
  	$output .= "<input type=\"".$this->elm_type."\" id=\"".$this->elm_id."\" name=\"".$this->elm_name."\" value=\"".$this->elm_value."\" class=\"".$this->elm_css_class."\" />".$this->elm_html_after;
  	return $output;
  }
}

class df_password extends df_textbox 
{
  protected function setup($value = NULL, $tabindex = NULL, $jscript = NULL)
  {
    $this->elm_type = "password";
    $this->elm_value = $value;
  }
}

class df_file extends df_textbox 
{
  protected function setup($value = NULL, $tabindex = NULL, $jscript = NULL)
  {
    $this->elm_type = "file";
    $this->elm_value = $value; 
  }
public function draw()
  {
    if($this->isDesignActive())
    {
      $this->addCSS("input_file"); 
    }
    $output = NULL;
    $output .= $this->elm_html_before;
    
    if($this->elm_label != NULL)
    {
      $output .= "<label for=\"".$this->elm_id."\">".$this->elm_label."</label>";
    }
    
    $output .= "<input type=\"".$this->elm_type."\" id=\"".$this->elm_id."\" name=\"".$this->elm_name."\" value=\"".$this->elm_value."\" class=\"".$this->elm_css_class."\" />".$this->elm_html_after;
    return $output;
  }
}

class df_hidden extends df_element
{
	/*
	 * this encapsulation is just done for better Intellisense support
	 */
  public function __construct($name, $id, $value)
  {
  	parent::__construct($name, $id, $value);
  }
  
	protected function setup($value = NULL, $tabindex = NULL, $jscript = NULL)
  {
    $this->elm_value = $value;
    $this->elm_type = "hidden";
  }
  
  public function draw()
  {
    $output = NULL;
    $output .= $this->elm_html_before."<input type=\"".$this->elm_type."\" id=\""
            .  $this->elm_id."\" name=\"".$this->elm_name."\" value=\""
            .  $this->elm_value."\" />".$this->elm_html_after;
    
    return $output;
  }
}

class df_button extends df_element
{
  protected function setup($value = NULL, $tabindex = NULL, $jscript = NULL)
  {
    $this->elm_type = "button";
    $this->elm_value = $value;
  }
  
  public function draw()
  {
    if($this->isDesignActive())
    {
      $this->elm_css_class .= " input_button";
    }
  	$output = NULL;
    $output .= $this->elm_html_before;
    
    $output .= "<input type=\"".$this->elm_type."\" id=\"".$this->elm_id."\" name=\"".$this->elm_name."\" value=\"".$this->elm_value."\" class=\"".$this->elm_css_class."\" />".$this->elm_html_after;
    return $output;
  }
}

class df_submit extends df_button
{
  protected function setup($value = NULL, $tabindex = NULL, $jscript = NULL)
  {
    $this->elm_type = "submit";
    $this->elm_value = $value;
  }
}

class df_image extends df_element
{
  protected $elm_src = NULL;
  
	public function __construct($src, $name, $id=NULL, $value=NULL, $label=NULL, $css_class=NULL, $tabindex=NULL, $jscript=NULL)
  {
    parent::__construct($name, $id, $value, $label, $css_class, $tabindex, $jscript);
    
    $this->elm_src = $src;
  }
  
	protected function setup($value = NULL, $tabindex = NULL, $jscript = NULL)
  {
    $this->elm_type = "image";
    $this->elm_value = $value;
  }
  
  public function draw()
  {
    if($this->isDesignActive())
    {
      $this->addCSS("input_image");
    }
    $output = NULL;
    $output .= $this->elm_html_before;
    
    $output .= "<input type=\"".$this->elm_type."\" src=\"".$this->elm_src."\" id=\"".$this->elm_id."\" name=\"".$this->elm_name."\" class=\"".$this->elm_css_class."\" />".$this->elm_html_after;
    return $output;
  }
}

class df_reset extends df_button
{
  protected function setup($value = NULL, $tabindex = NULL, $jscript = NULL)
  {
    $this->elm_type = "reset";
    $this->elm_value = $value;
  }
}

class df_textarea extends df_element
{
  protected function setup($value = NULL, $tabindex = NULL, $jscript = NULL)
  {
    $this->elm_type = "textarea";
    $this->elm_value = $value;
  }
  
  public function draw()
  {
    if($this->isDesignActive())
    {
      $this->addCSS("input_textarea");
    }
    
  
    $output = NULL;
    
    $output .= $this->elm_html_before;
    if($this->elm_label != NULL)
    {
      $output .= "<label for=\"".$this->elm_id."\">".$this->elm_label."</label>";
    }
    
    $output .= "<textarea id=\"".$this->elm_id."\" name=\"".$this->elm_name."\" class=\"".$this->elm_css_class."\" />".$this->elm_value."</textarea>".$this->elm_html_after;
    return $output;
  }
}

class df_checkbox extends df_element
{
	protected $elm_checked = false;
	
  public function __construct($name, $id=NULL, $value=NULL, $label=NULL, $checked = false, $css_class=NULL, $tabindex=NULL, $jscript=NULL)
  {
    parent::__construct($name, $id, $value, $label, $css_class, $tabindex, $jscript);
    
    $this->elm_checked = $checked;
  }
  
  protected function setup($value = NULL, $tabindex = NULL, $jscript = NULL)
  {
    $this->elm_type = "checkbox";
    $this->elm_value = $value;
  }
  
  public function draw()
  {
    if($this->isDesignActive())
    {
      $this->addCSS("input_checkbox");
    }
    
  
    $output = NULL;
    
    $output .= $this->elm_html_before;
    if($this->elm_label != NULL)
    {
      $output .= "<label for=\"".$this->elm_id."\">".$this->elm_label."</label>";
    }
    
    ($this->elm_checked == true) ? ($check = "checked") : ($check = "");
    
    $output .= "<input type=\"checkbox\" ".$check." id=\"".$this->elm_id."\" name=\"".$this->elm_name."\" class=\"".$this->elm_css_class."\" value=\"".$this->elm_value."\" />".$this->elm_html_after;
    return $output;
  }
}

class df_dropdown extends df_element
{
  const VALUE_PARAMS = 2;
	protected function setup($value = NULL, $tabindex = NULL, $jscript = NULL)
  {
    if(gettype($value) != "array")
    {
    	throw new ex_unvalidValueParam("Dropdown needs following Array structure as Value: array ( array(value, display ), ... )");
    }
    else
    {
	    foreach($value as $item)
	    {
	    	if(count($item) > self::VALUE_PARAMS)
	    	{
	    	  throw new ex_unvalidValueParam("Dropdowns Array structure may only have 2 params in subarray");	
	    	}
	    }
    }
    
  	$this->elm_type = "dropdown";
    $this->elm_value = $value;
  }
  
  public function draw()
  {
    if($this->isDesignActive())
    {
      $this->addCSS("input_dropdown");
    }
    
  
    $output = NULL;
    
    $output .= $this->elm_html_before;
    if($this->elm_label != NULL)
    {
      $output .= "<label for=\"".$this->elm_id."\">".$this->elm_label."</label>";
    }
    $output .= "<select id=\"".$this->elm_id."\" name=\"".$this->elm_name."\" class=\"".$this->elm_css_class."\">";
    
    foreach($this->elm_value as $elem)
    {
    	if((count($elem) == 1) && (strtolower($elem[0]) != "end"))
    	{
    		$output .= "<OPTGROUP LABEL=\"".$elem[0]."\">";
    	}
    	elseif((count($elem[0]) == 1) && (strtolower($elem[0]) == "end"))
    	{
    		$output .= "</OPTGROUP>";
    	}
    	else
    	{
    	  $output .= "<option value=\"".$elem[0]."\">".$elem[1]."</option>";	
    	}
    }
    
    $output .= "</select>".$this->elm_html_after;
   
    return $output;
  }
}

class df_radio extends df_element
{
	protected $elm_checked   = false;
	protected $elm_groupName = NULL;
	
  public function __construct($name, $id=NULL, $value=NULL, $label=NULL, $checked = false, $groupName = NULL, $css_class=NULL, $tabindex=NULL, $jscript=NULL)
  {
  	parent::__construct($name, $id, $value, $label, $css_class, $tabindex, $jscript);
  	
  	$this->elm_groupName = $groupName;
  	$this->elm_checked = $checked;
  }
  
  protected function setup($value = NULL, $tabindex = NULL, $jscript = NULL)
  {
    $this->elm_type = "radio";
    $this->elm_value = $value;
  }
  
  
  /*
   * check if there are other existing radioboxes in this group
   * because only one may be checked
   */
  protected function checkRadioGroup()
  {
  	if($this->elm_bound_to->checkGroup($this->elm_id, $this->elm_name, $this->elm_groupName) == true)
  	{
  		$this->elm_checked = false;
  	}
  }
  
  public function getGroupName()
  {
  	return $this->elm_groupName;
  }
  public function isChecked()
  {
  	return $this->elm_checked;
  }
  
  public function draw()
  {
    if($this->isDesignActive())
    {
      $this->addCSS("input_radio");
    }
    // look for only one checked radiobox
    $this->checkRadioGroup();
    
    $output = NULL;
    $output .= $this->elm_html_before;
    
    if($this->elm_label != NULL)
    {
      $output .= "<label for=\"".$this->elm_id."\">".$this->elm_label."</label>";
    }
    
    ($this->getGroupName() != NULL) ? ($gpname = $this->getGroupName()) : ($gpname = $this->elm_name); 
    ($this->elm_checked == true) ? ($checked = "checked=\"checked\"") : ($checked="");
    
    $output .= "<input ".$checked." type=\"".$this->elm_type."\" id=\"".$this->elm_id."\" class=\"".$this->elm_css_class."\" name=\"".$gpname."\" value=\"".$this->elm_value."\"> ".$this->elm_html_after;
    return $output;
  }
}

?>